# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io
import warnings

from pydantic.v1 import validate_arguments, ValidationError
from typing_extensions import Annotated
from typing import overload, Optional, Union, Awaitable

from pydantic.v1 import Field, StrictBytes, StrictStr

from typing import Optional, Union

from linebot.v3.messaging.models.get_message_content_transcoding_response import GetMessageContentTranscodingResponse

from linebot.v3.messaging.async_api_client import AsyncApiClient
from linebot.v3.messaging.api_response import ApiResponse
from linebot.v3.messaging.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class AsyncMessagingApiBlob(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = AsyncApiClient.get_default()
        self.api_client = api_client
        self.line_base_path = "https://api.line.me"


    @overload
    async def get_message_content(self, message_id : Annotated[StrictStr, Field(..., description="Message ID of video or audio")], **kwargs) -> bytearray:  # noqa: E501
        ...

    @overload
    def get_message_content(self, message_id : Annotated[StrictStr, Field(..., description="Message ID of video or audio")], async_req: Optional[bool]=True, **kwargs) -> bytearray:  # noqa: E501
        ...

    @validate_arguments
    def get_message_content(self, message_id : Annotated[StrictStr, Field(..., description="Message ID of video or audio")], async_req: Optional[bool]=None, **kwargs) -> Union[bytearray, Awaitable[bytearray]]:  # noqa: E501
        """get_message_content  # noqa: E501

        Download image, video, and audio data sent from users.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_message_content(message_id, async_req=True)
        >>> result = thread.get()

        :param message_id: Message ID of video or audio (required)
        :type message_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: bytearray
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_message_content_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.get_message_content_with_http_info(message_id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_message_content_with_http_info(self, message_id : Annotated[StrictStr, Field(..., description="Message ID of video or audio")], **kwargs) -> ApiResponse:  # noqa: E501
        """get_message_content  # noqa: E501

        Download image, video, and audio data sent from users.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_message_content_with_http_info(message_id, async_req=True)
        >>> result = thread.get()

        :param message_id: Message ID of video or audio (required)
        :type message_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(bytearray, status_code(int), headers(HTTPHeaderDict))
        """

        _hosts = [
            'https://api-data.line.me'
        ]
        _host = _hosts[0]
        if kwargs.get('_host_index'):
            _host_index = int(kwargs.get('_host_index'))
            if _host_index < 0 or _host_index >= len(_hosts):
                raise ApiValueError(
                    "Invalid host index. Must be 0 <= index < %s"
                    % len(_host)
                )
            _host = _hosts[_host_index]
        _params = locals()

        _all_params = [
            'message_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params and _key != "_host_index":
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_message_content" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['message_id']:
            _path_params['messageId'] = _params['message_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "bytearray",
        }

        return self.api_client.call_api(
            '/v2/bot/message/{messageId}/content', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def get_message_content_preview(self, message_id : Annotated[StrictStr, Field(..., description="Message ID of image or video")], **kwargs) -> bytearray:  # noqa: E501
        ...

    @overload
    def get_message_content_preview(self, message_id : Annotated[StrictStr, Field(..., description="Message ID of image or video")], async_req: Optional[bool]=True, **kwargs) -> bytearray:  # noqa: E501
        ...

    @validate_arguments
    def get_message_content_preview(self, message_id : Annotated[StrictStr, Field(..., description="Message ID of image or video")], async_req: Optional[bool]=None, **kwargs) -> Union[bytearray, Awaitable[bytearray]]:  # noqa: E501
        """get_message_content_preview  # noqa: E501

        Get a preview image of the image or video  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_message_content_preview(message_id, async_req=True)
        >>> result = thread.get()

        :param message_id: Message ID of image or video (required)
        :type message_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: bytearray
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_message_content_preview_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.get_message_content_preview_with_http_info(message_id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_message_content_preview_with_http_info(self, message_id : Annotated[StrictStr, Field(..., description="Message ID of image or video")], **kwargs) -> ApiResponse:  # noqa: E501
        """get_message_content_preview  # noqa: E501

        Get a preview image of the image or video  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_message_content_preview_with_http_info(message_id, async_req=True)
        >>> result = thread.get()

        :param message_id: Message ID of image or video (required)
        :type message_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(bytearray, status_code(int), headers(HTTPHeaderDict))
        """

        _hosts = [
            'https://api-data.line.me'
        ]
        _host = _hosts[0]
        if kwargs.get('_host_index'):
            _host_index = int(kwargs.get('_host_index'))
            if _host_index < 0 or _host_index >= len(_hosts):
                raise ApiValueError(
                    "Invalid host index. Must be 0 <= index < %s"
                    % len(_host)
                )
            _host = _hosts[_host_index]
        _params = locals()

        _all_params = [
            'message_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params and _key != "_host_index":
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_message_content_preview" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['message_id']:
            _path_params['messageId'] = _params['message_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "bytearray",
        }

        return self.api_client.call_api(
            '/v2/bot/message/{messageId}/content/preview', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def get_message_content_transcoding_by_message_id(self, message_id : Annotated[StrictStr, Field(..., description="Message ID of video or audio")], **kwargs) -> GetMessageContentTranscodingResponse:  # noqa: E501
        ...

    @overload
    def get_message_content_transcoding_by_message_id(self, message_id : Annotated[StrictStr, Field(..., description="Message ID of video or audio")], async_req: Optional[bool]=True, **kwargs) -> GetMessageContentTranscodingResponse:  # noqa: E501
        ...

    @validate_arguments
    def get_message_content_transcoding_by_message_id(self, message_id : Annotated[StrictStr, Field(..., description="Message ID of video or audio")], async_req: Optional[bool]=None, **kwargs) -> Union[GetMessageContentTranscodingResponse, Awaitable[GetMessageContentTranscodingResponse]]:  # noqa: E501
        """get_message_content_transcoding_by_message_id  # noqa: E501

        Verify the preparation status of a video or audio for getting  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_message_content_transcoding_by_message_id(message_id, async_req=True)
        >>> result = thread.get()

        :param message_id: Message ID of video or audio (required)
        :type message_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetMessageContentTranscodingResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_message_content_transcoding_by_message_id_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.get_message_content_transcoding_by_message_id_with_http_info(message_id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_message_content_transcoding_by_message_id_with_http_info(self, message_id : Annotated[StrictStr, Field(..., description="Message ID of video or audio")], **kwargs) -> ApiResponse:  # noqa: E501
        """get_message_content_transcoding_by_message_id  # noqa: E501

        Verify the preparation status of a video or audio for getting  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_message_content_transcoding_by_message_id_with_http_info(message_id, async_req=True)
        >>> result = thread.get()

        :param message_id: Message ID of video or audio (required)
        :type message_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetMessageContentTranscodingResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _hosts = [
            'https://api-data.line.me'
        ]
        _host = _hosts[0]
        if kwargs.get('_host_index'):
            _host_index = int(kwargs.get('_host_index'))
            if _host_index < 0 or _host_index >= len(_hosts):
                raise ApiValueError(
                    "Invalid host index. Must be 0 <= index < %s"
                    % len(_host)
                )
            _host = _hosts[_host_index]
        _params = locals()

        _all_params = [
            'message_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params and _key != "_host_index":
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_message_content_transcoding_by_message_id" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['message_id']:
            _path_params['messageId'] = _params['message_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "GetMessageContentTranscodingResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/message/{messageId}/content/transcoding', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def get_rich_menu_image(self, rich_menu_id : Annotated[StrictStr, Field(..., description="ID of the rich menu with the image to be downloaded")], **kwargs) -> bytearray:  # noqa: E501
        ...

    @overload
    def get_rich_menu_image(self, rich_menu_id : Annotated[StrictStr, Field(..., description="ID of the rich menu with the image to be downloaded")], async_req: Optional[bool]=True, **kwargs) -> bytearray:  # noqa: E501
        ...

    @validate_arguments
    def get_rich_menu_image(self, rich_menu_id : Annotated[StrictStr, Field(..., description="ID of the rich menu with the image to be downloaded")], async_req: Optional[bool]=None, **kwargs) -> Union[bytearray, Awaitable[bytearray]]:  # noqa: E501
        """get_rich_menu_image  # noqa: E501

        Download rich menu image.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_rich_menu_image(rich_menu_id, async_req=True)
        >>> result = thread.get()

        :param rich_menu_id: ID of the rich menu with the image to be downloaded (required)
        :type rich_menu_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: bytearray
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_rich_menu_image_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.get_rich_menu_image_with_http_info(rich_menu_id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_rich_menu_image_with_http_info(self, rich_menu_id : Annotated[StrictStr, Field(..., description="ID of the rich menu with the image to be downloaded")], **kwargs) -> ApiResponse:  # noqa: E501
        """get_rich_menu_image  # noqa: E501

        Download rich menu image.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_rich_menu_image_with_http_info(rich_menu_id, async_req=True)
        >>> result = thread.get()

        :param rich_menu_id: ID of the rich menu with the image to be downloaded (required)
        :type rich_menu_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(bytearray, status_code(int), headers(HTTPHeaderDict))
        """

        _hosts = [
            'https://api-data.line.me'
        ]
        _host = _hosts[0]
        if kwargs.get('_host_index'):
            _host_index = int(kwargs.get('_host_index'))
            if _host_index < 0 or _host_index >= len(_hosts):
                raise ApiValueError(
                    "Invalid host index. Must be 0 <= index < %s"
                    % len(_host)
                )
            _host = _hosts[_host_index]
        _params = locals()

        _all_params = [
            'rich_menu_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params and _key != "_host_index":
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_rich_menu_image" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['rich_menu_id']:
            _path_params['richMenuId'] = _params['rich_menu_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "bytearray",
        }

        return self.api_client.call_api(
            '/v2/bot/richmenu/{richMenuId}/content', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @overload
    async def set_rich_menu_image(self, rich_menu_id : Annotated[StrictStr, Field(..., description="The ID of the rich menu to attach the image to")], body : Optional[Union[StrictBytes, StrictStr]] = None, **kwargs) -> None:  # noqa: E501
        ...

    @overload
    def set_rich_menu_image(self, rich_menu_id : Annotated[StrictStr, Field(..., description="The ID of the rich menu to attach the image to")], body : Optional[Union[StrictBytes, StrictStr]] = None, async_req: Optional[bool]=True, **kwargs) -> None:  # noqa: E501
        ...

    @validate_arguments
    def set_rich_menu_image(self, rich_menu_id : Annotated[StrictStr, Field(..., description="The ID of the rich menu to attach the image to")], body : Optional[Union[StrictBytes, StrictStr]] = None, async_req: Optional[bool]=None, **kwargs) -> Union[None, Awaitable[None]]:  # noqa: E501
        """set_rich_menu_image  # noqa: E501

        Upload rich menu image  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.set_rich_menu_image(rich_menu_id, body, async_req=True)
        >>> result = thread.get()

        :param rich_menu_id: The ID of the rich menu to attach the image to (required)
        :type rich_menu_id: str
        :param body:
        :type body: bytearray
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the set_rich_menu_image_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.set_rich_menu_image_with_http_info(rich_menu_id, body, **kwargs)  # noqa: E501

    @validate_arguments
    def set_rich_menu_image_with_http_info(self, rich_menu_id : Annotated[StrictStr, Field(..., description="The ID of the rich menu to attach the image to")], body : Optional[Union[StrictBytes, StrictStr]] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """set_rich_menu_image  # noqa: E501

        Upload rich menu image  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.set_rich_menu_image_with_http_info(rich_menu_id, body, async_req=True)
        >>> result = thread.get()

        :param rich_menu_id: The ID of the rich menu to attach the image to (required)
        :type rich_menu_id: str
        :param body:
        :type body: bytearray
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _hosts = [
            'https://api-data.line.me'
        ]
        _host = _hosts[0]
        if kwargs.get('_host_index'):
            _host_index = int(kwargs.get('_host_index'))
            if _host_index < 0 or _host_index >= len(_hosts):
                raise ApiValueError(
                    "Invalid host index. Must be 0 <= index < %s"
                    % len(_host)
                )
            _host = _hosts[_host_index]
        _params = locals()

        _all_params = [
            'rich_menu_id',
            'body'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params and _key != "_host_index":
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_rich_menu_image" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['rich_menu_id']:
            _path_params['richMenuId'] = _params['rich_menu_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['body'] is not None:
            _body_params = _params['body']
            # convert to byte array if the input is a file name (str)
            if isinstance(_body_params, str):
                with io.open(_body_params, "rb") as _fp:
                   _body_params_from_file = _fp.read()
                _body_params = _body_params_from_file

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/v2/bot/richmenu/{richMenuId}/content', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class AudienceGroupStatus(str, Enum):
    """
    Status
    """

    """
    allowed enum values
    """
    IN_PROGRESS = 'IN_PROGRESS'
    READY = 'READY'
    FAILED = 'FAILED'
    EXPIRED = 'EXPIRED'
    INACTIVE = 'INACTIVE'
    ACTIVATING = 'ACTIVATING'

    @classmethod
    def from_json(cls, json_str: str) -> AudienceGroupStatus:
        """Create an instance of AudienceGroupStatus from a JSON string"""
        return AudienceGroupStatus(json.loads(json_str))





# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictStr, constr
from linebot.v3.messaging.models.action import Action

class QuickReplyItem(BaseModel):
    """
    QuickReplyItem
    https://developers.line.biz/en/reference/messaging-api/#items-object
    """
    image_url: Optional[constr(strict=True, max_length=2000)] = Field(None, alias="imageUrl", description="URL of the icon that is displayed at the beginning of the button")
    action: Optional[Action] = None
    type: Optional[StrictStr] = Field('action', description="`action`")

    __properties = ["imageUrl", "action", "type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> QuickReplyItem:
        """Create an instance of QuickReplyItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of action
        if self.action:
            _dict['action'] = self.action.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> QuickReplyItem:
        """Create an instance of QuickReplyItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return QuickReplyItem.parse_obj(obj)

        _obj = QuickReplyItem.parse_obj({
            "image_url": obj.get("imageUrl"),
            "action": Action.from_dict(obj.get("action")) if obj.get("action") is not None else None,
            "type": obj.get("type") if obj.get("type") is not None else 'action'
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, Field, StrictBool, StrictInt, conlist
from linebot.v3.audience.models.audience_group import AudienceGroup

class GetSharedAudienceGroupsResponse(BaseModel):
    """
    Gets data for more than one audience.
    https://developers.line.biz/en/reference/messaging-api/#get-audience-groups
    """
    audience_groups: Optional[conlist(AudienceGroup)] = Field(None, alias="audienceGroups", description="An array of audience data. If there are no audiences that match the specified filter, an empty array will be returned.")
    has_next_page: Optional[StrictBool] = Field(None, alias="hasNextPage", description="true when this is not the last page.")
    total_count: Optional[StrictInt] = Field(None, alias="totalCount", description="The total number of audiences that can be returned with the specified filter.")
    read_write_audience_group_total_count: Optional[StrictInt] = Field(None, alias="readWriteAudienceGroupTotalCount", description="Of the audiences you can get with the specified filter, the number of audiences with the update permission set to READ_WRITE.")
    page: Optional[StrictInt] = Field(None, description="The current page number.")
    size: Optional[StrictInt] = Field(None, description="The maximum number of audiences on the current page.")

    __properties = ["audienceGroups", "hasNextPage", "totalCount", "readWriteAudienceGroupTotalCount", "page", "size"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetSharedAudienceGroupsResponse:
        """Create an instance of GetSharedAudienceGroupsResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in audience_groups (list)
        _items = []
        if self.audience_groups:
            for _item in self.audience_groups:
                if _item:
                    _items.append(_item.to_dict())
            _dict['audienceGroups'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetSharedAudienceGroupsResponse:
        """Create an instance of GetSharedAudienceGroupsResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetSharedAudienceGroupsResponse.parse_obj(obj)

        _obj = GetSharedAudienceGroupsResponse.parse_obj({
            "audience_groups": [AudienceGroup.from_dict(_item) for _item in obj.get("audienceGroups")] if obj.get("audienceGroups") is not None else None,
            "has_next_page": obj.get("hasNextPage"),
            "total_count": obj.get("totalCount"),
            "read_write_audience_group_total_count": obj.get("readWriteAudienceGroupTotalCount"),
            "page": obj.get("page"),
            "size": obj.get("size")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr

class GetWebhookEndpointResponse(BaseModel):
    """
    GetWebhookEndpointResponse
    https://developers.line.biz/en/reference/messaging-api/#get-webhook-endpoint-information
    """
    endpoint: StrictStr = Field(..., description="Webhook URL")
    active: StrictBool = Field(..., description="Webhook usage status. Send a webhook event from the LINE Platform to the webhook URL only if enabled.  `true`: Webhook usage is enabled. `false`: Webhook usage is disabled. ")

    __properties = ["endpoint", "active"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetWebhookEndpointResponse:
        """Create an instance of GetWebhookEndpointResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetWebhookEndpointResponse:
        """Create an instance of GetWebhookEndpointResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetWebhookEndpointResponse.parse_obj(obj)

        _obj = GetWebhookEndpointResponse.parse_obj({
            "endpoint": obj.get("endpoint"),
            "active": obj.get("active")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json
import linebot.v3.messaging.models


from typing import Union
from pydantic.v1 import BaseModel, Field, StrictStr

class SubstitutionObject(BaseModel):
    """
    An object that defines the replacement value for a placeholder in the text.
    """
    type: StrictStr = Field(..., description="Type of substitution object")

    __properties = ["type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    # JSON field name that stores the object type
    __discriminator_property_name = 'type'

    # discriminator mappings
    __discriminator_value_class_map = {
        'emoji': 'EmojiSubstitutionObject',
        'mention': 'MentionSubstitutionObject'
    }

    @classmethod
    def get_discriminator_value(cls, obj: dict) -> str:
        """Returns the discriminator value (object type) of the data"""
        discriminator_value = obj[cls.__discriminator_property_name]
        if discriminator_value:
            return cls.__discriminator_value_class_map.get(discriminator_value)
        else:
            return None

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Union(EmojiSubstitutionObject, MentionSubstitutionObject):
        """Create an instance of SubstitutionObject from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Union(EmojiSubstitutionObject, MentionSubstitutionObject):
        """Create an instance of SubstitutionObject from a dict"""
        # look up the object type based on discriminator mapping
        object_type = cls.get_discriminator_value(obj)
        if object_type:
            klass = getattr(linebot.v3.messaging.models, object_type)
            return klass.from_dict(obj)
        else:
            raise ValueError("SubstitutionObject failed to lookup discriminator value from " +
                             json.dumps(obj) + ". Discriminator property name: " + cls.__discriminator_property_name +
                             ", mapping: " + json.dumps(cls.__discriminator_value_class_map))




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.filter module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .base import Base


class Operator(with_metaclass(ABCMeta, Base)):
    """Operator.

    https://developers.line.biz/en/reference/messaging-api/#narrowcast-demographic-filter

    Use logical AND, OR, and NOT operators to combine multiple recipient objects or
    demographic filter objects together.
    You can specify up to 10 recipient objects or demographic filter objects per request.
    """

    def __init__(self, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(Operator, self).__init__(**kwargs)

        self.type = "operator"


class And(Operator):
    """And.

    Create a new recipient object or demographic filter object by taking the
    logical conjunction (AND) of the specified array of objects.
    """

    def __init__(self, *args, **kwargs):
        """__init__ method.

        :param args:
        :param kwargs:
        """
        super(And, self).__init__(**kwargs)

        setattr(self, 'and', args)


class Or(Operator):
    """Or.

    Create a new recipient object or demographic filter object by taking the
    logical disjunction (OR) of the specified array of objects.
    """

    def __init__(self, *args, **kwargs):
        """__init__ method.

        :param args:
        :param kwargs:
        """
        super(Or, self).__init__(**kwargs)

        setattr(self, 'or', args)


class Not(Operator):
    """Not.

    Create a new recipient object or demographic filter object that excludes
    in the specified object.
    """

    def __init__(self, arg, **kwargs):
        """__init__ method.

        :param arg:
        :param kwargs:
        """
        super(Not, self).__init__(**kwargs)

        setattr(self, 'not', arg)



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional, Union
from pydantic.v1 import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, conlist, validator

class Membership(BaseModel):
    """
    Membership
    """
    membership_id: StrictInt = Field(..., alias="membershipId", description="Membership plan ID.")
    title: StrictStr = Field(..., description="Membership plan name.")
    description: StrictStr = Field(..., description="Membership plan description.")
    benefits: conlist(StrictStr, min_items=1) = Field(..., description="List of membership plan perks.")
    price: Union[StrictFloat, StrictInt] = Field(..., description="Monthly fee for membership plan. (e.g. 1500.00)")
    currency: StrictStr = Field(..., description="The currency of membership.price.")
    member_count: StrictInt = Field(..., alias="memberCount", description="Number of members subscribed to the membership plan.")
    member_limit: Optional[StrictInt] = Field(..., alias="memberLimit", description="The upper limit of members who can subscribe. If no upper limit is set, it will be null.")
    is_in_app_purchase: StrictBool = Field(..., alias="isInAppPurchase", description="Payment method for users who subscribe to a membership plan.")
    is_published: StrictBool = Field(..., alias="isPublished", description="Membership plan status.")

    __properties = ["membershipId", "title", "description", "benefits", "price", "currency", "memberCount", "memberLimit", "isInAppPurchase", "isPublished"]

    @validator('currency')
    def currency_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('JPY', 'TWD', 'THB'):
            raise ValueError("must be one of enum values ('JPY', 'TWD', 'THB')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Membership:
        """Create an instance of Membership from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if member_limit (nullable) is None
        # and __fields_set__ contains the field
        if self.member_limit is None and "member_limit" in self.__fields_set__:
            _dict['memberLimit'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Membership:
        """Create an instance of Membership from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Membership.parse_obj(obj)

        _obj = Membership.parse_obj({
            "membership_id": obj.get("membershipId"),
            "title": obj.get("title"),
            "description": obj.get("description"),
            "benefits": obj.get("benefits"),
            "price": obj.get("price"),
            "currency": obj.get("currency"),
            "member_count": obj.get("memberCount"),
            "member_limit": obj.get("memberLimit"),
            "is_in_app_purchase": obj.get("isInAppPurchase"),
            "is_published": obj.get("isPublished")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API(Insight)

    This document describes LINE Messaging API(Insight).  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, Field, StrictBool, conlist
from linebot.v3.insight.models.age_tile import AgeTile
from linebot.v3.insight.models.app_type_tile import AppTypeTile
from linebot.v3.insight.models.area_tile import AreaTile
from linebot.v3.insight.models.gender_tile import GenderTile
from linebot.v3.insight.models.subscription_period_tile import SubscriptionPeriodTile

class GetFriendsDemographicsResponse(BaseModel):
    """
    Get friend demographics
    https://developers.line.biz/en/reference/messaging-api/#get-demographic
    """
    available: Optional[StrictBool] = Field(None, description="true if friend demographic information is available.")
    genders: Optional[conlist(GenderTile)] = Field(None, description="Percentage per gender.")
    ages: Optional[conlist(AgeTile)] = Field(None, description="Percentage per age group.")
    areas: Optional[conlist(AreaTile)] = Field(None, description="Percentage per area.")
    app_types: Optional[conlist(AppTypeTile)] = Field(None, alias="appTypes", description="Percentage by OS.")
    subscription_periods: Optional[conlist(SubscriptionPeriodTile)] = Field(None, alias="subscriptionPeriods", description="Percentage per friendship duration.")

    __properties = ["available", "genders", "ages", "areas", "appTypes", "subscriptionPeriods"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetFriendsDemographicsResponse:
        """Create an instance of GetFriendsDemographicsResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in genders (list)
        _items = []
        if self.genders:
            for _item in self.genders:
                if _item:
                    _items.append(_item.to_dict())
            _dict['genders'] = _items
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in ages (list)
        _items = []
        if self.ages:
            for _item in self.ages:
                if _item:
                    _items.append(_item.to_dict())
            _dict['ages'] = _items
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in areas (list)
        _items = []
        if self.areas:
            for _item in self.areas:
                if _item:
                    _items.append(_item.to_dict())
            _dict['areas'] = _items
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in app_types (list)
        _items = []
        if self.app_types:
            for _item in self.app_types:
                if _item:
                    _items.append(_item.to_dict())
            _dict['appTypes'] = _items
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in subscription_periods (list)
        _items = []
        if self.subscription_periods:
            for _item in self.subscription_periods:
                if _item:
                    _items.append(_item.to_dict())
            _dict['subscriptionPeriods'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetFriendsDemographicsResponse:
        """Create an instance of GetFriendsDemographicsResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetFriendsDemographicsResponse.parse_obj(obj)

        _obj = GetFriendsDemographicsResponse.parse_obj({
            "available": obj.get("available"),
            "genders": [GenderTile.from_dict(_item) for _item in obj.get("genders")] if obj.get("genders") is not None else None,
            "ages": [AgeTile.from_dict(_item) for _item in obj.get("ages")] if obj.get("ages") is not None else None,
            "areas": [AreaTile.from_dict(_item) for _item in obj.get("areas")] if obj.get("areas") is not None else None,
            "app_types": [AppTypeTile.from_dict(_item) for _item in obj.get("appTypes")] if obj.get("appTypes") is not None else None,
            "subscription_periods": [SubscriptionPeriodTile.from_dict(_item) for _item in obj.get("subscriptionPeriods")] if obj.get("subscriptionPeriods") is not None else None
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import json
import unittest

import responses

from linebot import (
    LineBotApi
)
from linebot.models import (
    URITemplateAction,
    RichMenu, RichMenuSize, RichMenuArea, RichMenuBounds
)
from linebot.models.actions import RichMenuSwitchAction


class TestLineBotApi(unittest.TestCase):
    maxDiff = None

    def setUp(self):
        self.tested = LineBotApi('channel_secret')

        self.rich_menu_id = 'richmenu-0000000000'
        self.user_id = 'userid'
        self.rich_menu = RichMenu(
            size=RichMenuSize(
                width=2500,
                height=1686
            ),
            selected=False,
            name="nice richmenu",
            chatBarText="touch me",
            areas=[
                RichMenuArea(
                    RichMenuBounds(
                        x=0,
                        y=0,
                        width=833,
                        height=843
                    ),
                    URITemplateAction(
                        uri='https://line.me/R/nv/location/'
                    )
                )
            ]
        )

    @responses.activate
    def test_get_rich_menu(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/rich_menu_id',
            json={
                "richMenuId": "rich_menu_id",
                "size": {
                    "width": 2500,
                    "height": 1686
                },
                "selected": False,
                "name": "name",
                "chatBarText": "chatBarText",
                "areas": [
                    {
                        "bounds": {
                            "x": 0,
                            "y": 0,
                            "width": 2500,
                            "height": 1686
                        },
                        "action": {
                            "type": "postback",
                            "data": "action=buy&itemid=123"
                        }
                    }
                ]
            },
            status=200
        )

        rich_menu = self.tested.get_rich_menu('rich_menu_id')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/rich_menu_id'
        )

        self.assertEqual(rich_menu.rich_menu_id, 'rich_menu_id')
        self.assertEqual(rich_menu.size.width, 2500)
        self.assertEqual(rich_menu.size.height, 1686)
        self.assertEqual(rich_menu.selected, False)
        self.assertEqual(rich_menu.name, 'name')
        self.assertEqual(rich_menu.chat_bar_text, 'chatBarText')
        self.assertEqual(rich_menu.areas[0].bounds.x, 0)
        self.assertEqual(rich_menu.areas[0].bounds.y, 0)
        self.assertEqual(rich_menu.areas[0].bounds.width, 2500)
        self.assertEqual(rich_menu.areas[0].bounds.height, 1686)
        self.assertEqual(rich_menu.areas[0].action.type, 'postback')
        self.assertEqual(rich_menu.areas[0].action.data, 'action=buy&itemid=123')

    @responses.activate
    def test_validate_rich_menu_object(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/validate',
            json={}, status=200
        )

        rich_menu = RichMenu(
            size=RichMenuSize(
                width=2500,
                height=1686
            ),
            selected=False,
            name="nice richmenu",
            chatBarText="touch me",
            areas=[
                RichMenuArea(
                    RichMenuBounds(
                        x=0,
                        y=0,
                        width=833,
                        height=843
                    ),
                    URITemplateAction(
                        uri='https://line.me/R/nv/location/'
                    )
                )
            ]
        )

        self.tested.validate_rich_menu_object(rich_menu)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/validate'
        )

    @responses.activate
    def test_create_rich_menu(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu',
            json={"richMenuId": "richMenuId"}, status=200
        )

        rich_menu = RichMenu(
            size=RichMenuSize(
                width=2500,
                height=1686
            ),
            selected=False,
            name="nice richmenu",
            chatBarText="touch me",
            areas=[
                RichMenuArea(
                    RichMenuBounds(
                        x=0,
                        y=0,
                        width=833,
                        height=843
                    ),
                    URITemplateAction(
                        uri='https://line.me/R/nv/location/'
                    )
                )
            ]
        )

        result = self.tested.create_rich_menu(rich_menu)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu'
        )
        self.assertEqual(result, "richMenuId")

    @responses.activate
    def test_delete_rich_menu(self):
        responses.add(
            responses.DELETE,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/rich_menu_id',
            json={}, status=200
        )

        self.tested.delete_rich_menu('rich_menu_id')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'DELETE')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/rich_menu_id'
        )

    @responses.activate
    def test_get_rich_menu_id_of_user(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/user/user_id/richmenu',
            json={"richMenuId": "richMenuId"}, status=200
        )

        result = self.tested.get_rich_menu_id_of_user('user_id')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/user/user_id/richmenu'
        )
        self.assertEqual(result, "richMenuId")

    @responses.activate
    def test_link_rich_menu_to_user(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/user/user_id/richmenu/rich_menu_id',
            json={}, status=200
        )

        self.tested.link_rich_menu_to_user('user_id', 'rich_menu_id')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/user/user_id/richmenu/rich_menu_id'
        )

    @responses.activate
    def test_unlink_rich_menu_from_user(self):
        responses.add(
            responses.DELETE,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/user/user_id/richmenu',
            json={}, status=200
        )

        self.tested.unlink_rich_menu_from_user('user_id')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'DELETE')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/user/user_id/richmenu'
        )

    @responses.activate
    def test_get_rich_menu_list(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/list',
            json={
                "richmenus": [
                    {
                        "richMenuId": "rich_menu_id",
                        "size": {
                            "width": 2500,
                            "height": 1686
                        },
                        "selected": False,
                        "name": "name",
                        "chatBarText": "chatBarText",
                        "areas": [
                            {
                                "bounds": {
                                    "x": 0,
                                    "y": 0,
                                    "width": 2500,
                                    "height": 1686
                                },
                                "action": {
                                    "type": "postback",
                                    "data": "action=buy&itemid=123"
                                }
                            }
                        ]
                    }
                ]
            },
            status=200
        )

        rich_menus = self.tested.get_rich_menu_list()

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/list'
        )
        self.assertEqual(rich_menus[0].rich_menu_id, 'rich_menu_id')
        self.assertEqual(rich_menus[0].size.width, 2500)
        self.assertEqual(rich_menus[0].size.height, 1686)
        self.assertEqual(rich_menus[0].selected, False)
        self.assertEqual(rich_menus[0].name, 'name')
        self.assertEqual(rich_menus[0].chat_bar_text, 'chatBarText')
        self.assertEqual(rich_menus[0].areas[0].bounds.x, 0)
        self.assertEqual(rich_menus[0].areas[0].bounds.y, 0)
        self.assertEqual(rich_menus[0].areas[0].bounds.width, 2500)
        self.assertEqual(rich_menus[0].areas[0].bounds.height, 1686)
        self.assertEqual(rich_menus[0].areas[0].action.type, 'postback')
        self.assertEqual(rich_menus[0].areas[0].action.data, 'action=buy&itemid=123')

    @responses.activate
    def test_link_rich_menu_to_users(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/richmenu/bulk/link',
            json={}, status=202
        )

        self.tested.link_rich_menu_to_users(['user_id1', 'user_id2'], 'rich_menu_id')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/bulk/link'
        )
        self.assertEqual(
            json.loads(request.body),
            {
                "richMenuId": "rich_menu_id",
                "userIds": ["user_id1", "user_id2"],
            }
        )

    @responses.activate
    def test_unlink_rich_menu_to_users(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/richmenu/bulk/unlink',
            json={}, status=202
        )

        self.tested.unlink_rich_menu_from_users(['user_id1', 'user_id2'], 'rich_menu_id')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/bulk/unlink'
        )
        self.assertEqual(
            json.loads(request.body),
            {
                "userIds": ["user_id1", "user_id2"],
            }
        )

    @responses.activate
    def test_set_default_rich_menu(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/user/all/richmenu/rich_menu_id',
            json={}, status=200
        )

        self.tested.set_default_rich_menu('rich_menu_id')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/user/all/richmenu/rich_menu_id'
        )

    @responses.activate
    def test_get_default_rich_menu(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/user/all/richmenu',
            json={"richMenuId": "richMenuId"}, status=200
        )

        result = self.tested.get_default_rich_menu()

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/user/all/richmenu'
        )
        self.assertEqual(result, "richMenuId")

    @responses.activate
    def test_cancel_default_rich_menu(self):
        responses.add(
            responses.DELETE,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/user/all/richmenu',
            json={}, status=200
        )

        self.tested.cancel_default_rich_menu()

        request = responses.calls[0].request
        self.assertEqual(request.method, 'DELETE')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/user/all/richmenu'
        )

    @responses.activate
    def test_get_rich_menu_image(self):
        rich_menu_id = '1234'
        body = b'hogieoidksk'
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_DATA_ENDPOINT +
            '/v2/bot/richmenu/{rich_menu_id}/content'.format(rich_menu_id=rich_menu_id),
            body=body, status=200
        )

        res = self.tested.get_rich_menu_image(rich_menu_id)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_DATA_ENDPOINT +
            '/v2/bot/richmenu/{rich_menu_id}/content'.format(rich_menu_id=rich_menu_id),
        )
        self.assertEqual(
            body,
            res.content
        )

    @responses.activate
    def test_set_rich_menu_image(self):
        rich_menu_id = '1234'
        body = b'hogieoidksk'
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_DATA_ENDPOINT +
            '/v2/bot/richmenu/{rich_menu_id}/content'.format(rich_menu_id=rich_menu_id),
            json={}, status=200
        )

        self.tested.set_rich_menu_image(
            rich_menu_id=rich_menu_id,
            content_type='image/jpeg',
            content=body
        )

        request = responses.calls[0].request
        self.assertEqual('POST', request.method)
        self.assertEqual(
            LineBotApi.DEFAULT_API_DATA_ENDPOINT +
            '/v2/bot/richmenu/{rich_menu_id}/content'.format(rich_menu_id=rich_menu_id),
            request.url
        )

    @responses.activate
    def test_rich_menu_with_switch_action(self):
        rich_menu = RichMenu(
            size=RichMenuSize(
                width=2500,
                height=1686
            ),
            selected=False,
            name="nice richmenu",
            chatBarText="touch me",
            areas=[
                RichMenuArea(
                    RichMenuBounds(
                        x=0,
                        y=0,
                        width=833,
                        height=843
                    ),
                    RichMenuSwitchAction(
                        rich_menu_alias_id="richmenu-alias-a",
                        data="richmenu-changed-to-a"
                    )
                )
            ]
        )
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu',
            json={
                "richMenuId": "rich_menu_id",
                "size": {
                    "width": 2500,
                    "height": 1686
                },
                "selected": False,
                "name": "name",
                "chatBarText": "chatBarText",
                "areas": [
                    {
                        "bounds": {
                            "x": 0,
                            "y": 0,
                            "width": 2500,
                            "height": 1686
                        },
                        "action": {
                            "type": "richmenuswitch",
                            "richMenuAliasId": "richmenu-alias-a",
                            "data": "richmenu-changed-to-a"
                        }
                    }
                ]
            },
            status=200
        )

        result = self.tested.create_rich_menu(rich_menu)
        print(rich_menu)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu'
        )

        self.assertEqual(result, "rich_menu_id")


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic.v1 import Field
from linebot.v3.webhooks.models.chat_control import ChatControl
from linebot.v3.webhooks.models.delivery_context import DeliveryContext
from linebot.v3.webhooks.models.event import Event
from linebot.v3.webhooks.models.event_mode import EventMode
from linebot.v3.webhooks.models.source import Source

class ActivatedEvent(Event):
    """
    This event indicates that the module channel has been switched to Active Channel by calling the Acquire Control API. Sent to the webhook URL server of the module channel.
    """
    chat_control: ChatControl = Field(..., alias="chatControl")
    type: str = "activated"

    __properties = ["type", "source", "timestamp", "mode", "webhookEventId", "deliveryContext", "chatControl"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ActivatedEvent:
        """Create an instance of ActivatedEvent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of source
        if self.source:
            _dict['source'] = self.source.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of delivery_context
        if self.delivery_context:
            _dict['deliveryContext'] = self.delivery_context.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of chat_control
        if self.chat_control:
            _dict['chatControl'] = self.chat_control.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ActivatedEvent:
        """Create an instance of ActivatedEvent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ActivatedEvent.parse_obj(obj)

        _obj = ActivatedEvent.parse_obj({
            "type": obj.get("type"),
            "source": Source.from_dict(obj.get("source")) if obj.get("source") is not None else None,
            "timestamp": obj.get("timestamp"),
            "mode": obj.get("mode"),
            "webhook_event_id": obj.get("webhookEventId"),
            "delivery_context": DeliveryContext.from_dict(obj.get("deliveryContext")) if obj.get("deliveryContext") is not None else None,
            "chat_control": ChatControl.from_dict(obj.get("chatControl")) if obj.get("chatControl") is not None else None
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.v3.webhook module."""


import base64
import hashlib
import hmac
import inspect
import json

from .exceptions import InvalidSignatureError
from .webhooks import (
    Event,
    MessageEvent,
)
from .models.events import UnknownEvent ## Special
from .utils import LOGGER, PY3, safe_compare_digest

if hasattr(hmac, "compare_digest"):
    def compare_digest(val1, val2):
        """compare_digest function.

        If hmac module has compare_digest function, use it.
        Or not, use linebot.v3.utils.safe_compare_digest.

        :param val1: string or bytes for compare
        :type val1: str | bytes
        :param val2: string or bytes for compare
        :type val2: str | bytes
        :rtype: bool
        :return: result
        """
        return hmac.compare_digest(val1, val2)
else:
    def compare_digest(val1, val2):
        """compare_digest function.

        If hmac module has compare_digest function, use it.
        Or not, use linebot.v3.utils.safe_compare_digest.

        :param val1: string or bytes for compare
        :type val1: str | bytes
        :param val2: string or bytes for compare
        :type val2: str | bytes
        :rtype: bool
        :return: result
        """
        return safe_compare_digest(val1, val2)


class SignatureValidator(object):
    """Signature validator.

    https://developers.line.biz/en/reference/messaging-api/#signature-validation
    """

    def __init__(self, channel_secret):
        """__init__ method.

        :param str channel_secret: Channel secret (as text)
        """
        self.channel_secret = channel_secret.encode('utf-8')

    def validate(self, body, signature):
        """Check signature.

        :param str body: Request body (as text)
        :param str signature: X-Line-Signature value (as text)
        :rtype: bool
        """
        gen_signature = hmac.new(
            self.channel_secret,
            body.encode('utf-8'),
            hashlib.sha256
        ).digest()

        return compare_digest(
            signature.encode('utf-8'), base64.b64encode(gen_signature)
        )


class WebhookPayload(object):
    """Webhook Payload.

    https://developers.line.biz/en/reference/messaging-api/#request-body
    """

    def __init__(self, events=None, destination=None):
        """__init__ method.

        :param events: Information about the events.
        :type events: list[T <= :py:class:`linebot.v3.webhooks.models.Event`]
        :param str destination: User ID of a bot that should receive webhook events.
        """
        self.events = events
        self.destination = destination


class WebhookParser(object):
    """Webhook Parser."""

    def __init__(self, channel_secret):
        """__init__ method.

        :param str channel_secret: Channel secret (as text)
        """
        self.signature_validator = SignatureValidator(channel_secret)

    def parse(self, body, signature, as_payload=False):
        """Parse webhook request body as text.

        :param str body: Webhook request body (as text)
        :param str signature: X-Line-Signature value (as text)
        :param bool as_payload: (optional) True to return WebhookPayload object.
        :rtype: list[T <= :py:class:`linebot.v3.webhooks.models.Event`]
            | :py:class:`linebot.v3.webhook.WebhookPayload`
        :return: Events list, or WebhookPayload instance
        """
        if not self.signature_validator.validate(body, signature):
            raise InvalidSignatureError(
                'Invalid signature. signature=' + signature)

        body_json = json.loads(body)
        events = []
        for event in body_json['events']:
            try:
                events.append(Event.from_dict(event))
            except ValueError:
                LOGGER.info('Unknown event type. type=' + event['type'])
                events.append(UnknownEvent.new_from_json_dict(event))

        if as_payload:
            return WebhookPayload(events=events, destination=body_json.get('destination'))
        else:
            return events


class WebhookHandler(object):
    """Webhook Handler.

    Please read https://github.com/line/line-bot-sdk-python#webhookhandler
    """

    def __init__(self, channel_secret):
        """__init__ method.

        :param str channel_secret: Channel secret (as text)
        """
        self.parser = WebhookParser(channel_secret)
        self._handlers = {}
        self._default = None

    def add(self, event, message=None):
        """Add handler method.

        :param event: Specify a kind of Event which you want to handle
        :type event: T <= :py:class:`linebot.v3.webhooks.models.Event` class
        :param message: (optional) If event is MessageEvent,
            specify kind of Messages which you want to handle
        :type: message: T <= :py:class:`linebot.v3.webhooks.models.message_content.MessageContent` class
        :rtype: func
        :return: decorator
        """
        def decorator(func):
            if isinstance(message, (list, tuple)):
                for it in message:
                    self.__add_handler(func, event, message=it)
            else:
                self.__add_handler(func, event, message=message)

            return func

        return decorator

    def default(self):
        """Set default handler method.

        :rtype: func
        :return: decorator
        """
        def decorator(func):
            self._default = func
            return func

        return decorator

    def handle(self, body, signature):
        """Handle webhook.

        :param str body: Webhook request body (as text)
        :param str signature: X-Line-Signature value (as text)
        """
        payload = self.parser.parse(body, signature, as_payload=True)

        for event in payload.events:
            func = None
            key = None

            if isinstance(event, MessageEvent):
                key = self.__get_handler_key(
                    event.__class__, event.message.__class__)
                func = self._handlers.get(key, None)

            if func is None:
                key = self.__get_handler_key(event.__class__)
                func = self._handlers.get(key, None)

            if func is None:
                func = self._default

            if func is None:
                LOGGER.info('No handler of ' + key + ' and no default handler')
            else:
                self.__invoke_func(func, event, payload)

    def __add_handler(self, func, event, message=None):
        key = self.__get_handler_key(event, message=message)
        self._handlers[key] = func

    @classmethod
    def __invoke_func(cls, func, event, payload):
        (has_varargs, args_count) = cls.__get_args_count(func)
        if has_varargs or args_count == 2:
            func(event, payload.destination)
        elif args_count == 1:
            func(event)
        else:
            func()

    @staticmethod
    def __get_args_count(func):
        if PY3:
            arg_spec = inspect.getfullargspec(func)
            return (arg_spec.varargs is not None, len(arg_spec.args))
        else:
            arg_spec = inspect.getargspec(func)
            return (arg_spec.varargs is not None, len(arg_spec.args))

    @staticmethod
    def __get_handler_key(event, message=None):
        if message is None:
            return event.__name__
        else:
            return event.__name__ + '_' + message.__name__



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot package."""


from .__about__ import (  # noqa
    __version__
)
from .api import (  # noqa
    LineBotApi,
)
from .async_api import (  # noqa
    AsyncLineBotApi,
)
from .http_client import (  # noqa
    HttpClient,
    RequestsHttpClient,
    HttpResponse,
)
from .async_http_client import (  # noqa
    AsyncHttpClient,
    AsyncHttpResponse,
)
from .webhook import (  # noqa
    SignatureValidator,
    WebhookParser,
    WebhookHandler,
    WebhookPayload,
)

from .deprecations import (
    LineBotSdkDeprecationWarning,
    LineBotSdkDeprecatedIn30,
)



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic.v1 import BaseModel, Field
from linebot.v3.messaging.models.subscribed_membership_plan import SubscribedMembershipPlan
from linebot.v3.messaging.models.subscribed_membership_user import SubscribedMembershipUser

class Subscription(BaseModel):
    """
    An array of memberships.
    """
    membership: SubscribedMembershipPlan = Field(...)
    user: SubscribedMembershipUser = Field(...)

    __properties = ["membership", "user"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Subscription:
        """Create an instance of Subscription from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of membership
        if self.membership:
            _dict['membership'] = self.membership.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of user
        if self.user:
            _dict['user'] = self.user.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Subscription:
        """Create an instance of Subscription from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Subscription.parse_obj(obj)

        _obj = Subscription.parse_obj({
            "membership": SubscribedMembershipPlan.from_dict(obj.get("membership")) if obj.get("membership") is not None else None,
            "user": SubscribedMembershipUser.from_dict(obj.get("user")) if obj.get("user") is not None else None
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.send_messages module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .emojis import Emojis
from .actions import get_action
from .base import Base


class SendMessage(with_metaclass(ABCMeta, Base)):
    """Abstract Base Class of SendMessage."""

    def __init__(self, quick_reply=None, sender=None, **kwargs):
        """__init__ method.

        :param quick_reply: QuickReply object
        :type quick_reply: T <= :py:class:`linebot.models.send_messages.QuickReply`
        :param sender: Sender object
        :type sender: T <= :py:class:`linebot.models.send_messages.Sender`
        :param kwargs:
        """
        super(SendMessage, self).__init__(**kwargs)

        self.type = None
        self.quick_reply = self.get_or_new_from_json_dict(quick_reply, QuickReply)
        self.sender = self.get_or_new_from_json_dict(sender, Sender)


class TextSendMessage(SendMessage):
    """TextSendMessage.

    https://developers.line.biz/en/reference/messaging-api/#text-message
    """

    def __init__(self, text=None, emojis=None, quick_reply=None, sender=None, **kwargs):
        """__init__ method.

        :param str text: Message text
        :param quick_reply: QuickReply object
        :type quick_reply: T <= :py:class:`linebot.models.send_messages.QuickReply`
        :type sender: T <= :py:class:`linebot.models.send_messages.Sender`
        :param kwargs:
        """
        super(TextSendMessage, self).__init__(quick_reply=quick_reply, sender=sender, **kwargs)

        self.type = 'text'
        self.text = text
        if emojis:
            new_emojis = []
            for emoji in emojis:
                emoji_object = self.get_or_new_from_json_dict(
                    emoji, Emojis
                )
                if emoji_object:
                    new_emojis.append(emoji_object)
            self.emojis = new_emojis
        else:
            self.emojis = emojis


class ImageSendMessage(SendMessage):
    """ImageSendMessage.

    https://developers.line.biz/en/reference/messaging-api/#image-message
    """

    def __init__(self, original_content_url=None, preview_image_url=None,
                 quick_reply=None, sender=None, **kwargs):
        """__init__ method.

        :param str original_content_url: Image URL.
            HTTPS
            JPEG
            Max: 1024 x 1024
            Max: 1 MB
        :param str preview_image_url: Preview image URL
            HTTPS
            JPEG
            Max: 240 x 240
            Max: 1 MB
        :param quick_reply: QuickReply object
        :type quick_reply: T <= :py:class:`linebot.models.send_messages.QuickReply`
        :type sender: T <= :py:class:`linebot.models.send_messages.Sender`
        :param kwargs:
        """
        super(ImageSendMessage, self).__init__(quick_reply=quick_reply, sender=sender, **kwargs)

        self.type = 'image'
        self.original_content_url = original_content_url
        self.preview_image_url = preview_image_url


class VideoSendMessage(SendMessage):
    """VideoSendMessage.

    https://developers.line.biz/en/reference/messaging-api/#video-message
    """

    def __init__(self, original_content_url=None, preview_image_url=None,
                 tracking_id=None, quick_reply=None, sender=None, **kwargs):
        """__init__ method.

        :param str original_content_url: URL of video file.
            HTTPS. mp4. Less than 1 minute. Max: 10 MB.
        :param str preview_image_url: URL of preview image.
            HTTPS. JPEG. Max: 240 x 240. Max: 1 MB.
        :param str tracking_id: the video viewing complete event occurs
            when the user finishes watching the video.
            Max character limit: 100.
        :param quick_reply: QuickReply object
        :type quick_reply: T <= :py:class:`linebot.models.send_messages.QuickReply`
        :type sender: T <= :py:class:`linebot.models.send_messages.Sender`
        :param kwargs:
        """
        super(VideoSendMessage, self).__init__(quick_reply=quick_reply, sender=sender, **kwargs)

        self.type = 'video'
        self.original_content_url = original_content_url
        self.preview_image_url = preview_image_url
        self.tracking_id = tracking_id


class AudioSendMessage(SendMessage):
    """AudioSendMessage.

    https://developers.line.biz/en/reference/messaging-api/#audio-message
    """

    def __init__(self, original_content_url=None, duration=None, quick_reply=None,
                 sender=None, **kwargs):
        """__init__ method.

        :param str original_content_url: URL of audio file. HTTPS.
            m4a. Less than 1 minute. Max 10 MB.
        :param long duration: Length of audio file (milliseconds).
        :param quick_reply: QuickReply object
        :type quick_reply: T <= :py:class:`linebot.models.send_messages.QuickReply`
        :type sender: T <= :py:class:`linebot.models.send_messages.Sender`
        :param kwargs:
        """
        super(AudioSendMessage, self).__init__(quick_reply=quick_reply, sender=sender, **kwargs)

        self.type = 'audio'
        self.original_content_url = original_content_url
        self.duration = duration


class LocationSendMessage(SendMessage):
    """LocationSendMessage.

    https://developers.line.biz/en/reference/messaging-api/#location-message
    """

    def __init__(self, title=None, address=None, latitude=None, longitude=None,
                 quick_reply=None, sender=None, **kwargs):
        """__init__ method.

        :param str title: Title
        :param str address: Address
        :param float latitude: Latitude
        :param float longitude: Longitude
        :param quick_reply: QuickReply object
        :type quick_reply: T <= :py:class:`linebot.models.send_messages.QuickReply`
        :type sender: T <= :py:class:`linebot.models.send_messages.Sender`
        :param kwargs:
        """
        super(LocationSendMessage, self).__init__(quick_reply=quick_reply, sender=sender, **kwargs)

        self.type = 'location'
        self.title = title
        self.address = address
        self.latitude = latitude
        self.longitude = longitude


class StickerSendMessage(SendMessage):
    """StickerSendMessage.

    https://developers.line.biz/en/reference/messaging-api/#sticker-message
    """

    def __init__(self, package_id=None, sticker_id=None, quick_reply=None, sender=None, **kwargs):
        """__init__ method.

        :param str package_id: Package ID
        :param str sticker_id: Sticker ID
        :param quick_reply: QuickReply object
        :type quick_reply: T <= :py:class:`linebot.models.send_messages.QuickReply`
        :type sender: T <= :py:class:`linebot.models.send_messages.Sender`
        :param kwargs:
        """
        super(StickerSendMessage, self).__init__(quick_reply=quick_reply, sender=sender, **kwargs)

        self.type = 'sticker'
        self.package_id = package_id
        self.sticker_id = sticker_id


class QuickReply(with_metaclass(ABCMeta, Base)):
    """QuickReply.

    https://developers.line.me/en/docs/messaging-api/#quick-reply
    """

    def __init__(self, items=None, **kwargs):
        """__init__ method.

        :param items: Quick reply button objects
        :type items: list[T <= :py:class:`linebot.models.send_messages.QuickReplyButton`]
        :param kwargs:
        """
        super(QuickReply, self).__init__(**kwargs)

        new_items = []
        if items:
            for item in items:
                new_items.append(self.get_or_new_from_json_dict(
                    item, QuickReplyButton
                ))
        self.items = new_items


class QuickReplyButton(with_metaclass(ABCMeta, Base)):
    """QuickReplyButton.

    https://developers.line.me/en/reference/messaging-api/#quick-reply-button-object
    """

    def __init__(self, image_url=None, action=None, **kwargs):
        """__init__ method.

        :param str image_url: URL of the icon that is displayed
            at the beginning of the button
        :param action: Action performed when this button is tapped
        :type action: T <= :py:class:`linebot.models.actions.Action`
        :param kwargs:
        """
        super(QuickReplyButton, self).__init__(**kwargs)

        self.type = 'action'
        self.image_url = image_url
        self.action = get_action(action)


class Sender(with_metaclass(ABCMeta, Base)):
    """Sender.

    https://developers.line.biz/en/reference/messaging-api/#icon-nickname-switch
    """

    def __init__(self, name=None, icon_url=None, **kwargs):
        """__init__ method.

        :param str name: Display name
        :param str icon_url: Icon image URL
        """
        super(Sender, self).__init__(**kwargs)

        self.name = name
        self.icon_url = icon_url



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import pytest

from aiohttp import web

from linebot import (
    AsyncLineBotApi
)
from linebot.aiohttp_async_http_client import AiohttpAsyncHttpClient


@pytest.mark.asyncio
async def test_get(aiohttp_client):
    msg = ''.join('Hello, world' for i in range(1000))

    async def hello(request):
        return web.Response(text=msg)

    app = web.Application()
    app.router.add_get('//v2/bot/message/abc/content', hello)

    aiohttp = await aiohttp_client(app, server_kwargs={"skip_url_asserts": True})
    async_client = AiohttpAsyncHttpClient(session=aiohttp)

    bot = AsyncLineBotApi('TOKENTOKEN', async_client, data_endpoint='/')
    content = await bot.get_message_content("abc")
    got = b''
    async for s in content.iter_content():
        got += s
    assert got.decode('utf-8') == msg



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr

class SubscribedMembershipUser(BaseModel):
    """
    Object containing user membership subscription information.
    """
    membership_no: StrictInt = Field(..., alias="membershipNo", description="The user's member number in the membership plan.")
    joined_time: StrictInt = Field(..., alias="joinedTime", description="UNIX timestamp at which the user subscribed to the membership.")
    next_billing_date: StrictStr = Field(..., alias="nextBillingDate", description="Next payment date for membership plan. - Format: yyyy-MM-dd (e.g. 2024-02-08) - Timezone: UTC+9 ")
    total_subscription_months: StrictInt = Field(..., alias="totalSubscriptionMonths", description="The period of time in months that the user has been subscribed to a membership plan. If a user previously canceled and then re-subscribed to the same membership plan, only the period after the re-subscription will be counted.")

    __properties = ["membershipNo", "joinedTime", "nextBillingDate", "totalSubscriptionMonths"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SubscribedMembershipUser:
        """Create an instance of SubscribedMembershipUser from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SubscribedMembershipUser:
        """Create an instance of SubscribedMembershipUser from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SubscribedMembershipUser.parse_obj(obj)

        _obj = SubscribedMembershipUser.parse_obj({
            "membership_no": obj.get("membershipNo"),
            "joined_time": obj.get("joinedTime"),
            "next_billing_date": obj.get("nextBillingDate"),
            "total_subscription_months": obj.get("totalSubscriptionMonths")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictBool, conint

class AcquireChatControlRequest(BaseModel):
    """
    Request entity of the Acquire Control API
    https://developers.line.biz/en/reference/partner-docs/#acquire-control-api
    """
    expired: Optional[StrictBool] = Field(None, description="`True`: After the time limit (ttl) has passed, the initiative (Chat Control) will return to the Primary Channel. (Default) `False`: There's no time limit and the initiative (Chat Control) doesn't change over time. ")
    ttl: Optional[conint(strict=True, le=31536000)] = Field(None, description="The time it takes for initiative (Chat Control) to return to the Primary Channel (the time that the module channel stays on the Active Channel). The value is specified in seconds. The maximum value is one year (3600 * 24 * 365). The default value is 3600 (1 hour).  * Ignored if the value of expired is false. ")

    __properties = ["expired", "ttl"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AcquireChatControlRequest:
        """Create an instance of AcquireChatControlRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AcquireChatControlRequest:
        """Create an instance of AcquireChatControlRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AcquireChatControlRequest.parse_obj(obj)

        _obj = AcquireChatControlRequest.parse_obj({
            "expired": obj.get("expired"),
            "ttl": obj.get("ttl")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

import responses

from linebot import (
    LineBotApi
)


class TestLineBotApi(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')

    @responses.activate
    def test_get_number_of_units_used_this_month(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/aggregation/info',
            json={
                'numOfCustomAggregationUnits': 22
            },
            status=200
        )

        usage = self.tested.get_number_of_units_used_this_month()

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/aggregation/info')
        self.assertEqual(usage.num_of_custom_aggregation_units, 22)

    @responses.activate
    def test_get_name_list_of_units_used_this_month(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/aggregation/list',
            json={
                'customAggregationUnits': ['promotion_a', 'promotion_b']
            },
            status=200
        )

        name_list = self.tested.get_name_list_of_units_used_this_month(30)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/aggregation/list?limit=30')
        self.assertEqual(name_list.custom_aggregation_units,
                         ['promotion_a', 'promotion_b'])
        self.assertEqual(name_list.next, None)

    @responses.activate
    def test_get_name_list_of_units_used_this_month_with_start(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/aggregation/list',
            json={
                'customAggregationUnits': ['promotion_a', 'promotion_b'],
                'next': 'continuationToken2'
            },
            status=200
        )

        name_list = self.tested.get_name_list_of_units_used_this_month(start='continuationToken1')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/message/aggregation/list?limit=100&start=continuationToken1')
        self.assertEqual(name_list.custom_aggregation_units, ['promotion_a', 'promotion_b'])
        self.assertEqual(name_list.next, 'continuationToken2')

    @responses.activate
    def test_get_name_list_of_units_used_this_month_with_start_and_limit(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/aggregation/list',
            json={
                'customAggregationUnits': ['promotion_a', 'promotion_b'],
                'next': 'continuationToken2'
            },
            status=200
        )

        name_list = self.tested.get_name_list_of_units_used_this_month(limit=2,
                                                                       start='continuationToken1')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/message/aggregation/list?limit=2&start=continuationToken1')
        self.assertEqual(name_list.custom_aggregation_units,
                         ['promotion_a', 'promotion_b'])
        self.assertEqual(name_list.next, 'continuationToken2')


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictStr, validator

class BeaconContent(BaseModel):
    """
    BeaconContent
    """
    hwid: StrictStr = Field(..., description="Hardware ID of the beacon that was detected")
    type: StrictStr = Field(..., description="Type of beacon event.")
    dm: Optional[StrictStr] = Field(None, description="Device message of beacon that was detected.")

    __properties = ["hwid", "type", "dm"]

    @validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('enter', 'banner', 'stay'):
            raise ValueError("must be one of enum values ('enter', 'banner', 'stay')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> BeaconContent:
        """Create an instance of BeaconContent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> BeaconContent:
        """Create an instance of BeaconContent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return BeaconContent.parse_obj(obj)

        _obj = BeaconContent.parse_obj({
            "hwid": obj.get("hwid"),
            "type": obj.get("type"),
            "dm": obj.get("dm")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import json
import unittest

import responses

from linebot import (
    LineBotApi
)
from linebot.models import (
    TextSendMessage, QuickReply, QuickReplyButton,
    PostbackAction, MessageAction, DatetimePickerAction,
    CameraAction, CameraRollAction, LocationAction
)


class TestLineBotApi(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')

        # test data
        self.text_message = TextSendMessage(text='Hello, world')
        self.message = [{"type": "text", "text": "Hello, world"}]

    @responses.activate
    def test_push_text_message_with_quick_reply(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push',
            json={}, status=200
        )

        self.tested.push_message('to',
                                 TextSendMessage(
                                     text='Hello, world',
                                     quick_reply=QuickReply(items=[
                                         QuickReplyButton(
                                             image_url='https://example.com',
                                             action=PostbackAction(label="label1", data="data1")
                                         ),
                                         QuickReplyButton(
                                             action=MessageAction(label="label2", text="text2")
                                         ),
                                         QuickReplyButton(
                                             action=DatetimePickerAction(label="label3",
                                                                         data="data3",
                                                                         mode="date")
                                         ),
                                         QuickReplyButton(
                                             action=CameraAction(label="label4")
                                         ),
                                         QuickReplyButton(
                                             action=CameraRollAction(label="label5")
                                         ),
                                         QuickReplyButton(
                                             action=LocationAction(label="label6")
                                         ),
                                     ])))

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": "to",
                'notificationDisabled': False,
                "messages": [
                    {
                        "type": "text",
                        "text": "Hello, world",
                        "quickReply": {
                            "items": [
                                {
                                    "type": "action",
                                    "imageUrl": "https://example.com",
                                    "action": {
                                        "type": "postback",
                                        "label": "label1",
                                        "data": "data1",
                                    }
                                },
                                {
                                    "type": "action",
                                    "action": {
                                        "type": "message",
                                        "label": "label2",
                                        "text": "text2",
                                    }
                                },
                                {
                                    "type": "action",
                                    "action": {
                                        "type": "datetimepicker",
                                        "label": "label3",
                                        "data": "data3",
                                        "mode": "date",
                                    }
                                },
                                {
                                    "type": "action",
                                    "action": {
                                        "type": "camera",
                                        "label": "label4",
                                    }
                                },
                                {
                                    "type": "action",
                                    "action": {
                                        "type": "cameraRoll",
                                        "label": "label5",
                                    }
                                },
                                {
                                    "type": "action",
                                    "action": {
                                        "type": "location",
                                        "label": "label6",
                                    }
                                },
                            ]
                        }
                    }
                ]
            }
        )


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr, conlist
from linebot.v3.messaging.models.message import Message

class PushMessageRequest(BaseModel):
    """
    PushMessageRequest
    https://developers.line.biz/en/reference/messaging-api/#send-push-message
    """
    to: StrictStr = Field(..., description="ID of the receiver.")
    messages: conlist(Message, max_items=5, min_items=1) = Field(..., description="List of Message objects.")
    notification_disabled: Optional[StrictBool] = Field(False, alias="notificationDisabled", description="`true`: The user doesn’t receive a push notification when a message is sent. `false`: The user receives a push notification when the message is sent (unless they have disabled push notifications in LINE and/or their device). The default value is false. ")
    custom_aggregation_units: Optional[conlist(StrictStr)] = Field(None, alias="customAggregationUnits", description="List of aggregation unit name. Case-sensitive. This functions can only be used by corporate users who have submitted the required applications. ")

    __properties = ["to", "messages", "notificationDisabled", "customAggregationUnits"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PushMessageRequest:
        """Create an instance of PushMessageRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in messages (list)
        _items = []
        if self.messages:
            for _item in self.messages:
                if _item:
                    _items.append(_item.to_dict())
            _dict['messages'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PushMessageRequest:
        """Create an instance of PushMessageRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PushMessageRequest.parse_obj(obj)

        _obj = PushMessageRequest.parse_obj({
            "to": obj.get("to"),
            "messages": [Message.from_dict(_item) for _item in obj.get("messages")] if obj.get("messages") is not None else None,
            "notification_disabled": obj.get("notificationDisabled") if obj.get("notificationDisabled") is not None else False,
            "custom_aggregation_units": obj.get("customAggregationUnits")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import Field, StrictStr, constr, validator
from linebot.v3.messaging.models.action import Action

class PostbackAction(Action):
    """
    PostbackAction
    """
    data: Optional[constr(strict=True, max_length=300, min_length=0)] = None
    display_text: Optional[StrictStr] = Field(None, alias="displayText")
    text: Optional[StrictStr] = None
    input_option: Optional[StrictStr] = Field(None, alias="inputOption")
    fill_in_text: Optional[StrictStr] = Field(None, alias="fillInText")
    type: str = "postback"

    __properties = ["type", "label", "data", "displayText", "text", "inputOption", "fillInText"]

    @validator('input_option')
    def input_option_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('closeRichMenu', 'openRichMenu', 'openKeyboard', 'openVoice'):
            raise ValueError("must be one of enum values ('closeRichMenu', 'openRichMenu', 'openKeyboard', 'openVoice')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PostbackAction:
        """Create an instance of PostbackAction from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PostbackAction:
        """Create an instance of PostbackAction from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PostbackAction.parse_obj(obj)

        _obj = PostbackAction.parse_obj({
            "type": obj.get("type"),
            "label": obj.get("label"),
            "data": obj.get("data"),
            "display_text": obj.get("displayText"),
            "text": obj.get("text"),
            "input_option": obj.get("inputOption"),
            "fill_in_text": obj.get("fillInText")
        })
        return _obj

